﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.Reflection;

namespace GodHand.Infrastructure.Repository
{
    public class ConnectionFactory
    {
        static Dictionary<string, Type> cache = new Dictionary<string, Type>();
        static object lockHelper = new object();

        public static IDbConnection Create(string name)
        {
            var values = name.Split(';');
            var typeName = values.First();
            var assemblyName = values.Last();
            Type contextType = null;
            if (!cache.ContainsKey(name))
            {
                lock (lockHelper)
                {
                    if (!cache.ContainsKey(name))
                    {
                        if (values.Length == 1)
                        {
                            contextType = Type.GetType(typeName);
                        }
                        else
                        {
                            var assembly = Assembly.Load(assemblyName);
                            if (assembly == null) { throw new Exception(string.Format("未找到程序集{0}失败。", assemblyName)); }
                            contextType = assembly.GetTypes().FirstOrDefault(c => c.FullName == typeName);
                            if (contextType == null) { throw new Exception(string.Format("未找到DbConnection上下文类型{0}。", typeName)); }
                        }
                        cache.Add(name, contextType);
                    }
                }
            }
            contextType = cache[name];
            var context = Activator.CreateInstance(contextType) as IDbConnection;
            if (context == null) { throw new Exception(string.Format("未能创建DbConnection上下文对象{0}", typeName)); }
            return context;
        }
    }
}
