﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GodHand.Infrastructure.LinqParser;
using GodHand.Infrastructure.Repository;
using System.Data;

namespace GodHand.Infrastructure.BlueBubble
{
    public class DbContext : IDisposable, IObjectContext
    {
        public EntityStateManager Manager { get; private set; }
        public string DbType { get; private set; }
        public string ConnectionString { get; set; }
        public int PoolSize { get; set; }
        public bool AllowUpdateWithNoExp { get; set; }
        public bool AllowDeleteWithNoExp { get; set; }

        List<System.Data.IDbConnection> connPool = new List<System.Data.IDbConnection>();

        public DbContext()
        {
            var dbType = System.Configuration.ConfigurationManager.AppSettings["DbType"];
            var connectionString = System.Configuration.ConfigurationManager.ConnectionStrings["DbConnectionString"].ConnectionString;
            var poolSize = System.Configuration.ConfigurationManager.AppSettings["DbConnectionPoolSize"].To<int>(0);
            InitContext(connectionString, dbType, poolSize, true, true);
        }
        public DbContext(string connectionString, string dbType)
        {
            InitContext(connectionString, dbType, 30, true, true);
        }
        public DbContext(string connectionString, string dbType, bool allowUpdateWithNoExp, bool allowDeleteWithNoExp)
        {
            InitContext(connectionString, dbType, 30, allowUpdateWithNoExp, allowDeleteWithNoExp);
        }

        void InitContext(string connectionString, string dbType, int poolSize, bool allowUpdateWithNoExp, bool allowDeleteWithNoExp)
        {
            if (string.IsNullOrWhiteSpace(dbType) || string.IsNullOrWhiteSpace(connectionString)) { throw new ArgumentNullException("can't find DbType or DbConnectionString in appsetting."); }
            this.PoolSize = poolSize <= 0 ? 30 : poolSize;
            this.DbType = dbType;
            this.ConnectionString = connectionString;
            this.AllowDeleteWithNoExp = allowDeleteWithNoExp;
            this.AllowUpdateWithNoExp = allowUpdateWithNoExp;
            this.Manager = new EntityStateManager();
        }

        public EntitySet<T> GetEntitySet<T>(string tableName = null, bool noLock = false, bool noTracking = false)
        {
            return new EntitySet<T>(tableName, noLock, noTracking, this);
        }
        public int SubmitChange()
        {
            var ret = 0;
            using (var conn = this.CreateConnection())
            {
                conn.UsingTransaction(tran =>
                {
                    //this.Manager.InsertEntityCommands.ForEach(c =>
                    //{
                    //    ret += c.Execute(conn, tran);
                    //});
                    //this.Manager.InsertEntityCommands.Clear();
                    //this.Manager.UpdateEntityCommands.ForEach(c =>
                    //{
                    //    ret += c.Execute(conn, tran);
                    //});
                    //this.Manager.UpdateEntityCommands.Clear();
                    //this.Manager.DeleteEntityCommands.ForEach(c =>
                    //{
                    //    ret += c.Execute(conn, tran);
                    //});
                    //this.Manager.DeleteEntityCommands.Clear();
                    this.Manager.EntityCommands.ForEach(c =>
                    {
                        ret += c.Execute(conn, tran);
                    });
                    this.Manager.EntityCommands.Clear();
                });
            }
            return ret;
        }

        void IDisposable.Dispose()
        {
            connPool.ForEach(c =>
            {
                try
                {
                    if (c.State != System.Data.ConnectionState.Closed) { c.Close(); }
                }
                catch { }
            });
            connPool.Clear();
        }

        string IObjectContext.Name { get; set; }

        bool IObjectContext.SaveChanges()
        {
            return this.SubmitChange() > 0;
        }

        public System.Data.IDbConnection CreateConnection()
        {
            var conn = connPool.Where(c => c.State == System.Data.ConnectionState.Closed).FirstOrDefault();
            if (conn == null)
            {
                if (connPool.Count <= this.PoolSize)
                {

                    conn = ConnectionFactory.Create(this.DbType);
                    conn.ConnectionString = this.ConnectionString;
                    connPool.Add(conn);
                }
            }
            return conn;
        }

        System.Data.IDbConnection IObjectContext.CreateConnection()
        {
            return this.CreateConnection();
        }
    }
}
