﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using GodHand.Infrastructure.LinqParser;
using System.Reflection;

namespace GodHand.Infrastructure.BlueBubble
{
    public class DbQueryProvider : IQueryProvider
    {
        public event EventHandler OnExecuted;
        DbContext Context { get; set; }
        string TableName { get; set; }
        bool NoLock { get; set; }
        bool NoTracking { get; set; }

        public DbQueryProvider(string tableName, bool noLock, bool noTracking, DbContext context)
        {
            if (context == null) { throw new ArgumentNullException(); }
            this.TableName = tableName;
            this.NoLock = noLock;
            this.NoTracking = noTracking;
            this.Context = context;
        }

        public virtual IQueryable<T> CreateQuery<T>(Expression expression)
        {
            return new EntitySet<T>(this.TableName, this.NoLock, this.NoTracking, expression, this.Context);
        }

        public virtual IQueryable CreateQuery(Expression expression)
        {
            Type elementType = TypeHelper.GetElementType(expression.Type);
            try
            {
                return (IQueryable)Activator.CreateInstance(typeof(EntitySet<>).MakeGenericType(elementType), new object[] { this.TableName, this.NoLock, this.NoTracking, expression, this.Context });
            }
            catch (TargetInvocationException tie)
            {
                throw tie.InnerException;
            }
        }

        public virtual T Execute<T>(Expression expression)
        {
            return (T)this.ExecuteExp(expression);
        }

        public virtual object Execute(Expression expression)
        {
            return this.ExecuteExp(expression);
        }

        public virtual object ExecuteExp(Expression expression)
        {
            DbExpressionVisitor visitor = new DbExpressionVisitor(this.TableName, this.NoLock);
            visitor.Visit(expression);
            var cmd = new SelectCommand(visitor);
            var result = cmd.Execute(this.Context.DbType, this.Context.ConnectionString);
            if (this.OnExecuted != null) { this.OnExecuted(result, null); }
            return result;
        }
    }
}
