﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GodHand.Infrastructure.Repository;
using System.Linq.Expressions;
using System.ComponentModel;

namespace GodHand.Infrastructure.BlueBubble
{
    public class DbRepository<T> : AbstractRepository<T> where T : class, new()
    {
        DbContext Context = null;
        EntitySet<T> EntitySet = null;
        string TableName { get; set; }
        bool NoLock { get; set; }
        bool NoTracking { get; set; }

        public DbRepository(IObjectContext context)
            : base(context)
        {
            this.Context = base.Context as DbContext;
            if (this.Context == null) { throw new Exception("GodEFRepository中上下文对象不是GodHand.Infrastructure.BlueBubble.DbContext对象。"); }
        }

        public override IRepository<T> AsTableName(string tableName)
        {
            var r = new DbRepository<T>(this.Context);
            r.TableName = tableName;
            r.NoLock = this.NoLock;
            r.NoTracking = this.NoTracking;
            return r;
        }
        public override IRepository<T> AsNoLock()
        {
            var r = new DbRepository<T>(this.Context);
            r.TableName = this.TableName;
            r.NoLock = true;
            r.NoTracking = this.NoTracking;
            return r;
        }
        public override IRepository<T> AsNoTracking()
        {
            var r = new DbRepository<T>(this.Context);
            r.TableName = this.TableName;
            r.NoLock = this.NoLock;
            r.NoTracking = true;
            return r;
        }

        public override bool New(T entity, bool INSERT_IDENTITY = false)
        {
            GetObjectSet().Add(entity);
            return true;
        }

        public override bool Update(T entity)
        {
            return true;
        }

        public override void Update(Expression<Func<T, bool>> exp, object obj)
        {
            GetObjectSet().Update(exp, obj);
        }

        public override bool Delete(T t)
        {
            GetObjectSet().Remove(t);
            return true;
        }

        public override void Delete(Expression<Func<T, bool>> exp)
        {
            GetObjectSet().Remove(exp);
        }

        public override bool VirtualDelete(T t)
        {
            return true;
        }

        protected override IQueryable<T> GetQueryable()
        {
            return GetObjectSet();
        }

        EntitySet<T> GetObjectSet()
        {
            if (this.EntitySet == null)
            {
                this.EntitySet = this.Context.GetEntitySet<T>(this.TableName, this.NoLock, this.NoTracking);
            }
            return this.EntitySet;
        }
    }
}
