﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Linq.Expressions;
using GodHand.Infrastructure.Repository;
using GodHand.Infrastructure.LinqParser;

namespace GodHand.Infrastructure.BlueBubble
{
    public class DeleteCommand : EntityCommand
    {
        string TableName { get; set; }
        Expression WhereExpression { get; set; }

        int DeleteType { get; set; }

        Type EntityType { get; set; }
        object Entity { get; set; }

        public DeleteCommand(object entity, string tableName)
        {
            if (entity == null) { throw new ArgumentNullException(); }
            this.Entity = entity;
            this.EntityType = this.Entity.GetType();
            this.TableName = tableName;
            this.DeleteType = 0;
        }

        public DeleteCommand(Type entityType, Expression whereExp, string tableName)
        {
            if (entityType == null) { throw new ArgumentNullException(); }
            this.EntityType = entityType;
            this.WhereExpression = whereExp;
            this.TableName = tableName;
            this.DeleteType = 1;
        }

        public override int Execute(IDbConnection conn, IDbTransaction tran)
        {
            switch (this.DeleteType)
            {
                case 0: { return this.ExecuteEntityDelete(conn, tran); }
                case 1: { return this.ExecuteExpDelete(conn, tran); }
                default: { throw new Exception("不正确的删除操作."); }
            }
        }
        int ExecuteEntityDelete(IDbConnection conn, IDbTransaction tran)
        {
            var tableAttr = this.EntityType.GetMetaData();
            if (string.IsNullOrWhiteSpace(this.TableName)) { this.TableName = tableAttr.TableName; }

            var builder = new StringBuilder();
            builder.AppendFormat("DELETE FROM [{0}] ", this.TableName);
            DynamicParameters param = new DynamicParameters();
            var keyColumns = tableAttr.GetColumns().Where(c => c.PrimaryKey).ToList();
            keyColumns.ForEach(c =>
            {
                param.Add(c.ColumnName, EmitHelper.GetValue(c.Name, this.Entity));
            });
            builder.AppendFormat(" WHERE {0}", string.Join(",", keyColumns.Select(c => string.Format("{0}=@{0}", c.ColumnName))));
            return conn.Execute(builder.ToString(), param, tran);
        }
        int ExecuteExpDelete(IDbConnection conn, IDbTransaction tran)
        {
            var tableAttr = this.EntityType.GetMetaData();
            if (string.IsNullOrWhiteSpace(this.TableName)) { this.TableName = tableAttr.TableName; }

            var builder = new StringBuilder();
            builder.AppendFormat("DELETE FROM [{0}] ", this.TableName);
            DynamicParameters param = new DynamicParameters();
            if (this.WhereExpression != null)
            {
                var parseContext = new ParserContext(this.EntityType);
                var whereVisitor = new WhereExpressionVisitor();
                whereVisitor.Parse(this.WhereExpression, parseContext);
                builder.AppendFormat(" WHERE {0}", whereVisitor.WhereBuilder.Replace(string.Format("[{0}]", parseContext.TableAlias), string.Format("[{0}]", this.TableName)));
                parseContext.DbParameters.ForEach(c => { param.Add(c.Key, c.Value); });
            }
            return conn.Execute(builder.ToString(), param, tran);
        }
    }
}
