﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GodHand.Infrastructure.Repository;
using System.Data;

namespace GodHand.Infrastructure.BlueBubble
{
    public class InsertCommand : EntityCommand
    {
        object Entity { get; set; }
        string TableName { get; set; }
        bool IDENTITY_INSERT { get; set; }

        public InsertCommand(object entity, string tableName) : this(entity, tableName, false) { }
        public InsertCommand(object entity, string tableName, bool IDENTITY_INSERT)
        {
            if (entity == null) { throw new ArgumentNullException(); }
            this.Entity = entity;
            this.TableName = tableName;
            this.IDENTITY_INSERT = IDENTITY_INSERT;
        }

        public override int Execute(IDbConnection conn, IDbTransaction tran)
        {
            var tableAttr = this.Entity.GetType().GetMetaData();
            if (string.IsNullOrWhiteSpace(this.TableName)) { this.TableName = tableAttr.TableName; }
            ColumnAttribute identityColumn = null;
            if (!IDENTITY_INSERT)
            {
                var identityCount = tableAttr.GetColumns().Where(c => c.Identity).Count();
                if (identityCount == 1)
                {
                    identityColumn = tableAttr.GetColumns().Where(c => c.Identity).FirstOrDefault();
                }
                else if (identityCount > 1) { throw new Exception("不支持多个列自增。"); }
            }
            var param = new DynamicParameters();
            var columnNames = new List<string>();
            tableAttr.GetColumns().ForEach(c =>
            {
                if (!IDENTITY_INSERT && c.Identity) { return; }
                columnNames.Add(c.ColumnName);
                param.Add(c.ColumnName, EmitHelper.GetValue(c.Name, this.Entity));
            });

            var builder = new StringBuilder();
            if (IDENTITY_INSERT) { builder.AppendFormat("SET IDENTITY_INSERT [{0}] ON;", this.TableName); }
            builder.AppendFormat("INSERT INTO [{0}]({1}) VALUES({2});", this.TableName, string.Join(",", columnNames), "@" + string.Join(",@", columnNames));
            if (identityColumn != null)
            {
                builder.Append("SELECT SCOPE_IDENTITY();");
            }
            if (IDENTITY_INSERT) { builder.AppendFormat("SET IDENTITY_INSERT [{0}] OFF;SELECT 0;", this.TableName); }

            var result = conn.Query(builder.ToString(), param, tran);
            if (identityColumn != null)
            {
                var v = (result.First() as IDictionary<string, object>).Values.First();
                switch (identityColumn.DbType)
                {
                    case DbType.Int16: { v = v.To<short>(); break; }
                    case DbType.Int32: { v = v.To<int>(); break; }
                    case DbType.Int64: { v = v.To<long>(); break; }
                    case DbType.Decimal: { v = v.To<decimal>(); break; }
                }
                EmitHelper.SetValue(identityColumn.Name, this.Entity, new object[] { v });
            }
            return 1;
        }
    }
}
