﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GodHand.Infrastructure;
using GodHand.Infrastructure.Repository;
using GodHand.Infrastructure.LinqParser;
using System.Data;

namespace GodHand.Infrastructure.BlueBubble
{
    public class SelectCommand
    {
        DbExpressionVisitor Visitor { get; set; }
        public SelectCommand(DbExpressionVisitor visitor)
        {
            if (visitor == null) { throw new ArgumentNullException("SelectCommand构造函数的参数visitor"); }
            Visitor = visitor;
        }
        public object Execute(string dbType, string connectionString)
        {
            object ret = null;
            using (var conn = ConnectionFactory.Create(dbType))
            {
                conn.ConnectionString = connectionString;
                conn.Open();
                var dbParams = Visitor.DbParameters.ToDictionary(c => c.Key, c => c.Value);
                switch (Visitor.TriggerQueryMethodName)
                {
                    case "count":
                    case "longcount":
                    case "min":
                    case "max":
                    case "sum":
                    case "average":
                        {
                            var result = conn.Query(Visitor.QuerySQL, dbParams);
                            if (result.Count() > 0) { ret = (result.First() as IDictionary<string, object>).Values.First(); } else { ret = 0; }
                            break;
                        }
                    case "contains":
                    case "any":
                        {
                            var result = conn.Query(Visitor.QuerySQL, dbParams);
                            ret = (result.First() as IDictionary<string, object>).Values.First().Equals(1) ? true : false; break;
                        }
                    case "distinct": { throw new Exception("暂不支持Distinct"); }
                    case "first":
                        {
                            var result = conn.Query(Visitor.QuerySQL, dbParams);
                            if (result.Count() == 0) { throw new Exception("Can't find data!"); }
                            var dr = Visitor.ConvertData(result);
                            ret = dr[0];
                            break;
                        }
                    case "firstordefault":
                        {
                            var result = conn.Query(Visitor.QuerySQL, dbParams);
                            if (result.Count() > 0) { var dr = Visitor.ConvertData(result); ret = dr[0]; }
                            break;
                        }
                    case "todictionary": { throw new Exception("暂不支持Dictionary"); }
                    default :
                        {
                            var result = conn.Query(Visitor.QuerySQL, dbParams);
                            var dr = Visitor.ConvertData(result);
                            ret = dr;
                            break;
                        }
                }
            }
            return ret;
        }
    }
}
