﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;

namespace GodHand.Infrastructure
{
    public class ConcurrentList<T>:ICollection<T>
    {
        List<T> list = null;
        object syncRoot = null;
        public ConcurrentList()
        {
            list = new List<T>();
            syncRoot = new object();
        }

        public void Add(T item)
        {
            lock (syncRoot)
            {
                list.Add(item);
            }
        }

        public void Clear()
        {
            lock (syncRoot)
            {
                list.Clear();
            }
        }

        public bool Contains(T item)
        {
            return list.Contains(item);
        }

        public void CopyTo(T[] array, int arrayIndex)
        {
            list.CopyTo(array, arrayIndex);
        }

        public int Count
        {
            get { return list.Count; }
        }

        public bool IsReadOnly
        {
            get { return false; }
        }

        public bool Remove(T item)
        {
            lock (syncRoot)
            {
                return list.Remove(item);
            }
        }

        public IEnumerator<T> GetEnumerator()
        {
            return list.GetEnumerator();
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return list.GetEnumerator();
        }
    }
}
