﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Configuration;
using System.Xml.Serialization;
using System.Xml;
using System.Collections.Specialized;

namespace GodHand.Infrastructure
{
    public class ConfigurationManagerHelper
    {
        public static T GetSection<T>(string sectionName) where T : class
        {
            return ConfigurationManager.GetSection(sectionName) as T;
        }
        public static object GetSection(string sectionName)
        {
            return ConfigurationManager.GetSection(sectionName);
        }
        public static string AppSettings(string key)
        {
            return ConfigurationManager.AppSettings[key];
        }
        public static ConnectionStringSettings ConnectionStrings(string key)
        {
            return ConfigurationManager.ConnectionStrings[key];
        }
        static IDictionary<string, NameValueCollection> nameValueConfigDic = null;
        public static NameValueCollection GetNameValueConfiguration(string sectionName)
        {
            if (nameValueConfigDic == null) { nameValueConfigDic = new Dictionary<string, NameValueCollection>(); }
            if (nameValueConfigDic.ContainsKey(sectionName)) { return nameValueConfigDic[sectionName]; }
            else
            {
                var r = ConfigurationManagerHelper.GetSection<NameValueCollection>(sectionName);
                nameValueConfigDic.Add(sectionName, r);
                return r;
            }
        }
    }
    public abstract class AbstractSerializerSectionHandler : IConfigurationSectionHandler
    {

        #region IConfigurationSectionHandler 成员

        public abstract object Create(object parent, object configContext, XmlNode section);

        #endregion

        protected string GetAttributeValue(string name, XmlNode node)
        {
            return node.Attributes[name] == null ? string.Empty : node.Attributes[name].Value;
        }
    }
    public class SerializerSectionHandler<T> : AbstractSerializerSectionHandler
    {
        public override object Create(object parent, object configContext, XmlNode section)
        {
            XmlSerializer serializer = new XmlSerializer(typeof(T));
            return serializer.Deserialize(new XmlNodeReader(section));
        }
    }
}
