﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;

namespace GodHand.Infrastructure
{
    [AttributeUsage(AttributeTargets.Class, AllowMultiple = false)]
    public class TableAttribute : Attribute
    {
        List<ColumnAttribute> columnCache = null;
        List<TableReferenceAttribute> refCache = null;

        public string Name { get; set; }
        public string Alias { get; set; }
        public string Mapping { get; set; }
        public string TableName { get { return string.IsNullOrEmpty(this.Alias) ? this.Name : this.Alias; } }
        void InitCache()
        {
            if (columnCache == null) { columnCache = new List<ColumnAttribute>(); }
            if (refCache == null) { refCache = new List<TableReferenceAttribute>(); }
        }
        public void Add(ColumnAttribute column)
        {
            InitCache();
            if (columnCache.Any(c => c.Name == column.Name)) { return; }
            columnCache.Add(column);
        }
        public void Remove(ColumnAttribute column)
        {
            InitCache();
            var o = columnCache.FirstOrDefault(c => c.Name == column.Name);
            if (o == null) { return; }
            columnCache.Remove(o);
        }
        public List<ColumnAttribute> GetColumns()
        {
            return columnCache;
        }
        public void Add(TableReferenceAttribute tableRef)
        {
            InitCache();
            if (refCache.Any(c => c.ReferenceType == tableRef.ReferenceType)) { return; }
            refCache.Add(tableRef);
        }
        public void Remove(TableReferenceAttribute tableRef)
        {
            InitCache();
            var o = refCache.FirstOrDefault(c => c.ReferenceType == tableRef.ReferenceType);
            if (o == null) { return; }
            refCache.Remove(o);
        }
        public List<TableReferenceAttribute> GetReferences()
        {
            return refCache;
        }

    }
    [AttributeUsage(AttributeTargets.Property, AllowMultiple = false)]
    public class ColumnAttribute : Attribute
    {
        public string Name { get; set; }
        public string Alias { get; set; }
        public string ColumnName { get { return string.IsNullOrEmpty(this.Alias) ? this.Name : this.Alias; } }
        public DbType DbType { get; set; }
        public bool PrimaryKey { get; set; }
        public bool Identity { get; set; }
    }
    [AttributeUsage(AttributeTargets.Property, AllowMultiple = false)]
    public class MetaDataIgnoreAttribute : Attribute
    {

    }
    [AttributeUsage(AttributeTargets.Property, AllowMultiple = false)]
    public class TableReferenceAttribute : Attribute
    {
        public string JoinType { get; set; }
        public Type ReferenceType { get; set; }
    }
}
