﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using System.Data;
using System.Xml;
using System.Xml.Linq;

namespace GodHand.Infrastructure
{
    public static class ReflectionExtension
    {
        public static T GetCustomAttribute<T>(this Type type, bool inherit = true) where T : Attribute
        {
            return (T)type.GetCustomAttributes(typeof(T), inherit).FirstOrDefault();
        }

        public static T GetCustomAttribute<T>(this MethodInfo method, bool inherit = true) where T : Attribute
        {
            return (T)method.GetCustomAttributes(typeof(T), inherit).FirstOrDefault();
        }

        public static T GetCustomAttribute<T>(this ParameterInfo parameter, bool inherit = true) where T : Attribute
        {
            return (T)parameter.GetCustomAttributes(typeof(T), inherit).FirstOrDefault();
        }

        public static T GetCustomAttribute<T>(this PropertyInfo property, bool inherit = true) where T : Attribute
        {
            return (T)property.GetCustomAttributes(typeof(T), inherit).FirstOrDefault();
        }

        public static T[] GetCustomAttributes<T>(this Type type, bool inherit = true) where T : Attribute
        {
            return (T[])type.GetCustomAttributes(typeof(T), inherit);
        }

        public static T[] GetCustomAttributes<T>(this MethodInfo method, bool inherit = true) where T : Attribute
        {
            return (T[])method.GetCustomAttributes(typeof(T), inherit);
        }
        public static T[] GetCustomAttributes<T>(this PropertyInfo property, bool inherit = true) where T : Attribute
        {
            return (T[])property.GetCustomAttributes(typeof(T), inherit);
        }
        static Dictionary<Type, TableAttribute> MetaDataCache = new Dictionary<Type, TableAttribute>();
        public static Dictionary<Type, DbType> DbTypeMapCache = new Dictionary<Type, DbType> 
        { 
            { typeof(ushort), DbType.UInt16 }, { typeof(ushort?), DbType.UInt16 },
            { typeof(uint), DbType.UInt32 }, { typeof(uint?), DbType.UInt32 },
            { typeof(ulong), DbType.UInt64 },{ typeof(ulong?), DbType.UInt64 },
            { typeof(short), DbType.Int16 }, { typeof(short?), DbType.Int16 },
            { typeof(int), DbType.Int32 }, { typeof(int?), DbType.Int32 },
            { typeof(long), DbType.Int64 },{ typeof(long?), DbType.Int64 },
             { typeof(double), DbType.Double } ,{ typeof(double?), DbType.Double },
            { typeof(float), DbType.Single },  { typeof(float?), DbType.Single },
            { typeof(decimal), DbType.Decimal },{ typeof(decimal?), DbType.Decimal },
             { typeof(DateTime), DbType.DateTime },  { typeof(DateTime?), DbType.DateTime },
             { typeof(DateTimeOffset), DbType.DateTimeOffset },  { typeof(DateTimeOffset?), DbType.DateTimeOffset },
             { typeof(TimeSpan), DbType.Time },  { typeof(TimeSpan?), DbType.Time },
             { typeof(bool), DbType.Boolean },{ typeof(bool?), DbType.Boolean },
             { typeof(Guid), DbType.Guid },{ typeof(Guid?), DbType.Guid },
             { typeof(byte), DbType.Byte },{ typeof(byte?), DbType.Byte },
             { typeof(char), DbType.StringFixedLength },{ typeof(char?), DbType.StringFixedLength },
             { typeof(sbyte), DbType.SByte },{ typeof(sbyte?), DbType.SByte },
            { typeof(string), DbType.String }, 
            { typeof(byte[]), DbType.Binary },
            { typeof(Object), DbType.Object }
        };
        public static TableAttribute GetMetaData(this Type type)
        {
            if (!type.IsClass) { throw new Exception("只支持在类上获取元数据！"); }
            if (MetaDataCache.ContainsKey(type))
            {
                return MetaDataCache[type];
            }
            var tableAttr = type.GetCustomAttribute<TableAttribute>(false);
            if (tableAttr == null || string.IsNullOrEmpty(tableAttr.Mapping))
            {
                tableAttr = type.GetAttributeMetaData();
            }
            else
            {
                tableAttr = type.GetEdmMetaData();
            }
            if (!MetaDataCache.ContainsKey(type)) { try { MetaDataCache.Add(type, tableAttr); } catch { } }
            return tableAttr;
        }
        static TableAttribute GetAttributeMetaData(this Type type)
        {
            var tableArrt = type.GetCustomAttribute<TableAttribute>(false);
            if (tableArrt == null) { tableArrt = new TableAttribute { }; }
            tableArrt.Name = type.Name;
            PropertyInfo[] pros = null;
            if (type.IsAnonymousType())
            {
                pros = type.GetProperties();
            }
            else
            {
                pros = type.GetProperties(BindingFlags.Public | BindingFlags.Instance | BindingFlags.GetProperty | BindingFlags.SetProperty).Where(c => c.CanWrite && c.CanRead && c.GetCustomAttributes(typeof(MetaDataIgnoreAttribute), true).Length == 0).ToArray();
            }
            foreach (var pro in pros)
            {
                if (pro.PropertyType.IsClass && pro.PropertyType != typeof(string) && !pro.PropertyType.IsArray && !pro.PropertyType.IsGenericType)
                {
                    var refArrt = pro.GetCustomAttribute<TableReferenceAttribute>(false);
                    if (refArrt == null)
                    {
                        refArrt = new TableReferenceAttribute { JoinType = "left", ReferenceType = pro.PropertyType };
                    }
                    tableArrt.Add(refArrt);
                }
                else if (DbTypeMapCache.ContainsKey(pro.PropertyType))
                {
                    var columnAttr = pro.GetCustomAttribute<ColumnAttribute>(false);
                    if (columnAttr == null)
                    {
                        columnAttr = new ColumnAttribute { };
                    }
                    columnAttr.Name = pro.Name;
                    if (DbTypeMapCache.ContainsKey(pro.PropertyType)) { columnAttr.DbType = DbTypeMapCache[pro.PropertyType]; }
                    tableArrt.Add(columnAttr);
                }
            }
            return tableArrt;
        }
        static TableAttribute GetEdmMetaData(this Type type)
        {
            var tableArrt = type.GetCustomAttribute<TableAttribute>(false);

            tableArrt.Name = type.Name;
            var mappingName = tableArrt.Mapping.Split(',').Last();
            var mslio = type.Assembly.GetManifestResourceStream(mappingName + ".msl");
            var mslxdoc = XDocument.Load(mslio);
            var mslNameSpace = mslxdoc.Root.Name.NamespaceName;
            var mappingEntitySetNode = (from el in mslxdoc.Root
                  .Element(XName.Get("EntityContainerMapping", mslNameSpace))
                  .Elements()
                                        where el.Attribute("Name").Value.Equals(tableArrt.Name)
                                        select el).FirstOrDefault();
            tableArrt.Alias = mappingEntitySetNode
                                      .Element(XName.Get("EntityTypeMapping", mslNameSpace))
                                      .Element(XName.Get("MappingFragment", mslNameSpace)).Attribute("StoreEntitySet").Value;

            var mappingProNodes = from mporel in mappingEntitySetNode
                                      .Element(XName.Get("EntityTypeMapping", mslNameSpace))
                                      .Element(XName.Get("MappingFragment", mslNameSpace))
                                      .Elements(XName.Get("ScalarProperty", mslNameSpace))
                                  select mporel;

            var csdlio = type.Assembly.GetManifestResourceStream(mappingName + ".csdl");
            var csdlxdoc = XDocument.Load(csdlio);
            var csdlNameSpace = csdlxdoc.Root.Name.NamespaceName;
            var csdlAnnotationNameSpace = csdlxdoc.Root.GetNamespaceOfPrefix("annotation").NamespaceName;
            var csEntityNode = (from el in csdlxdoc.Root
                  .Elements(XName.Get("EntityType", csdlNameSpace))
                                where el.Attribute("Name").Value.Equals(tableArrt.Alias)
                                select el).FirstOrDefault();
            if (csEntityNode == null) { throw new Exception(string.Format("未能在{0}csdl中找到EntityType{1}.", mappingName, tableArrt.Name)); }

            var proNodes = from proel in csEntityNode.Elements(XName.Get("Property", csdlNameSpace)) select proel;
            var keyNames = from proel in csEntityNode.Element(XName.Get("Key", csdlNameSpace)).Elements(XName.Get("PropertyRef", csdlNameSpace)) select proel.Attribute("Name").Value;


            foreach (var proNode in proNodes)
            {
                var columnAttr = new ColumnAttribute { Alias = proNode.Attribute("Name").Value };
                var identityAttr = proNode.Attribute(XName.Get("StoreGeneratedPattern", csdlAnnotationNameSpace));
                if (identityAttr != null && identityAttr.Value.ToLower() == "identity") { columnAttr.Identity = true; }
                if (keyNames.Contains(columnAttr.Alias)) { columnAttr.PrimaryKey = true; }

                var mappingPropertyNode = mappingProNodes.Where(c => c.Attribute("ColumnName").Value == columnAttr.Alias).FirstOrDefault();
                if (mappingPropertyNode != null)
                {
                    columnAttr.Name = mappingPropertyNode.Attribute("Name").Value;
                }
                else
                { columnAttr.Name = columnAttr.Alias; }
                var proInfo = type.GetProperty(columnAttr.Name);
                if (proInfo == null) { throw new Exception(string.Format("映射解析错误，类型{0}中没有找到属性{1}.", tableArrt.Name, columnAttr.Name)); }
                if (DbTypeMapCache.ContainsKey(proInfo.PropertyType)) { columnAttr.DbType = DbTypeMapCache[proInfo.PropertyType]; }
                tableArrt.Add(columnAttr);
            }
            return tableArrt;
        }
        static string GetXmlAttributeValue(string name, XmlNode node)
        {
            return node.Attributes[name] == null ? string.Empty : node.Attributes[name].Value;
        }
    }
}
