﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using System.Reflection;

namespace GodHand.Infrastructure.LinqParser
{
    public class EvalNumericExpressionVisitor : AbstractExpressionVisitor
    {
        ParserContext Context { get; set; }
        public StringBuilder WhereBuilder;
        //QueryClosure CurrQueryClosure { get; set; }

        //public EvalNumericExpressionVisitor(QueryClosure qc, ParserContext context)
        //{
        //    if (context == null) { throw new ArgumentNullException("context"); }
        //    if (qc == null) { throw new ArgumentNullException("qc"); }
        //    this.Context = context;
        //    this.CurrQueryClosure = qc;
        //}

        public void Parse(Expression exp,ParserContext context)
        {
            if (context == null) { throw new ArgumentNullException("context"); }
            this.Context = context;
            this.WhereBuilder = new StringBuilder();
            this.Visit(exp);
        }

        protected override Expression VisitMemberAccess(MemberExpression m)
        {
            this.VistOneCon(m);
            return base.VisitMemberAccess(m);
        }

        protected override Expression VisitBinary(BinaryExpression b)
        {
            switch (b.NodeType)
            {
                case ExpressionType.Add:
                case ExpressionType.AddChecked:
                case ExpressionType.Subtract:
                case ExpressionType.SubtractChecked:
                case ExpressionType.Multiply:
                case ExpressionType.MultiplyChecked:
                case ExpressionType.Divide:
                case ExpressionType.Modulo:
                    {
                        this.VistOneCon(b.Left);
                        switch (b.NodeType)
                        {
                            case ExpressionType.Add:
                            case ExpressionType.AddChecked:
                                { this.WhereBuilder.Append("+"); break; }
                            case ExpressionType.Subtract:
                            case ExpressionType.SubtractChecked:
                                { this.WhereBuilder.Append("-"); break; }
                            case ExpressionType.Multiply:
                            case ExpressionType.MultiplyChecked:
                                { this.WhereBuilder.Append("*"); break; }
                            case ExpressionType.Divide:
                                { this.WhereBuilder.Append("/"); break; }
                            case ExpressionType.Modulo:
                                { this.WhereBuilder.Append("%"); break; }
                        }
                        this.VistOneCon(b.Right);
                        break;
                    }
            }
            return base.VisitBinary(b);
        }
        void VistOneCon(Expression exp)
        {
            if (this.IsConstantExpression(exp))
            {
                this.VisitConstant(exp);
            }
            else if (exp is MemberExpression)
            {
                this.VisitMember(exp as MemberExpression);
            }
            else if (exp is LambdaExpression)
            {
                this.VistOneCon((exp as LambdaExpression).Body);
            }
            else { throw new Exception("未能解析求值表达式类型."); }
        }
        void VisitMember(MemberExpression mExp)
        {
            if (mExp == null) { return; }
            var proName = mExp.Member.Name;
            var originalName = this.Context.GetColumnName(proName);
            this.Context.AddSelectColumnName(originalName);
            this.WhereBuilder.AppendFormat("[{0}].[{1}]", this.Context.TableAlias, originalName);
        }
        void VisitConstant(Expression cExp)
        {
            if (cExp == null) { return; }
            this.WhereBuilder.Append(GetConstant(cExp));
        }
        bool IsConstantExpression(Expression exp)
        {
            if (exp is ConstantExpression) { return true; }
            if (exp is MemberExpression) { return IsConstantExpression((exp as MemberExpression).Expression); }
            return false;
        }
        object GetConstant(Expression exp, List<Tuple<string, MemberTypes>> pros = null)
        {
            if (pros == null) { pros = new List<Tuple<string, MemberTypes>>(); }
            if (exp is ConstantExpression)
            {
                var cExp = exp as ConstantExpression;
                if (pros.Count == 0) { return cExp.Value; }
                object value = cExp.Value;
                pros.ForEach(c =>
                {
                    switch (c.Item2)
                    {
                        case MemberTypes.Field: { value = value.GetType().GetField(c.Item1).GetValue(value); break; }
                        case MemberTypes.Property: { value = value.GetType().GetProperty(c.Item1).GetValue(value, null); break; }
                    }
                });
                return value;
            }
            else if (exp is MemberExpression)
            {
                var mExp = ((exp) as MemberExpression);
                var memberInfo = mExp.Member;
                pros.Insert(0, Tuple.Create(memberInfo.Name, memberInfo.MemberType));
                return GetConstant(mExp.Expression, pros);
            }
            return null;
        }
    }
}
