﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;

namespace GodHand.Infrastructure.LinqParser
{
    public class GroupByExpressionParser
    {
        public static void Parse(Expression ge, ParserContext context)
        {
            if (ge is UnaryExpression)
            {
                var operandExp = (ge as UnaryExpression).Operand;
                GroupByExpressionParser.Parse(operandExp, context);
            }
            else if (ge is LambdaExpression)
            {
                var lambdaExp = ge as LambdaExpression;
                GroupByExpressionParser.Parse(lambdaExp.Body,context);
            }
            else if (ge is MemberInitExpression)
            {
                var newExp = ge as MemberInitExpression;
                foreach (var b in newExp.Bindings)
                {
                    if (b is MemberAssignment)
                    {
                        var ma = (b as MemberAssignment);
                        var cName = ma.Member.Name;
                        var originalName = context.GetColumnName(cName);
                        context.AddSelectColumnName(originalName);
                        context.AddGroupBy(originalName);
                    }
                }
            }
            else if (ge is NewExpression)
            {
                var newExp = ge as NewExpression;
                for (var i = 0; i < newExp.Arguments.Count; i++)
                {
                    var arg = newExp.Arguments[i];

                    if (arg is MemberExpression)
                    {
                        var member = newExp.Members[i];
                        var aliasName = member.Name;
                        var columnName = (arg as MemberExpression).Member.Name;
                        if (aliasName != columnName)
                        {
                            context.AddColumnMap(aliasName, columnName);
                        }
                        var originalName = context.GetColumnName(columnName);
                        context.AddSelectColumnName(originalName);
                        context.AddGroupBy(originalName);
                    }
                }
            }
            else if (ge is MemberExpression)
            {
                var columnName = (ge as MemberExpression).Member.Name;
                var originalName = context.GetColumnName(columnName);
                context.AddSelectColumnName(originalName);
                context.AddGroupBy(originalName);
            }
            else { throw new Exception("未能解析Groupby Expression."); }
        }
    }
}
