﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GodHand.Infrastructure.LinqParser
{
    public static class LinqParseExtension
    {
        public static List<dynamic> ConvertData(this SelectTypeConstructor selectTypeCtor, IEnumerable<dynamic> sourceData, ParserContext context)
        {
            if (sourceData == null || context == null || sourceData.Count() == 0) { return new List<dynamic>(); }
            //var selectTypeCtor = new SelectTypeConstructor { SelectType = context.EntityType, ConstructorInfo = null };
            //if (this.SelectTypes.Count > 0) { selectTypeCtor = this.SelectTypes.First(); }

            var hasNonParamCtor = selectTypeCtor.SelectType.GetConstructors().Any(ic => ic.GetParameters().Length == 0);

            TableAttribute selectMetaData = null;
            List<dynamic> readerResult = new List<dynamic>();
            foreach (var s in sourceData)
            {
                var sidc = s as IDictionary<string, object>;
                dynamic instance = null;

                if (hasNonParamCtor)
                {
                    instance = Activator.CreateInstance(selectTypeCtor.SelectType);
                    if (selectMetaData == null) { selectMetaData = selectTypeCtor.SelectType.GetMetaData(); }
                    selectMetaData.GetColumns().ForEach(c =>
                    {
                        var proName = c.Name;
                        var colName = context.GetColumnName(proName);
                        var val = sidc[colName];
                        val = val == DBNull.Value ? null : val;
                        try
                        {
                            EmitHelper.SetValue(proName, instance, new object[] { val });
                        }
                        catch (Exception ex)
                        {
                            throw new Exception(string.Format("{0} of {1} is not nullable,but value is DbNull.{2}", proName, selectMetaData.Name, ex.Message));
                        }
                    });
                }
                else if (selectTypeCtor.ConstructorInfo != null)
                {
                    var ctorParams = new List<object> { };
                    selectTypeCtor.ParamNames.ForEach(pc =>
                    {
                        var proName = pc;
                        var colName = context.GetColumnName(proName);
                        var val = sidc[colName];
                        val = val == DBNull.Value ? null : val;
                        ctorParams.Add(val);
                    });
                    instance = selectTypeCtor.ConstructorInfo.Invoke(ctorParams.ToArray());
                }
                else
                {
                    instance = Activator.CreateInstance(context.EntityType);
                    var proName = selectTypeCtor.ParamNames.First();
                    var colName = context.GetColumnName(proName);
                    var val = sidc[colName];
                    val = val == DBNull.Value ? null : val;
                    try
                    {
                        EmitHelper.SetValue(proName, instance, new object[] { val });
                    }
                    catch (Exception ex)
                    {
                        throw new Exception(string.Format("{0} of {1} is not nullable,but value is DbNull.{2}", proName, selectMetaData.Name, ex.Message));
                    }
                }

                readerResult.Add(instance);
            }
            return readerResult;
        }
    }
}
