﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;

namespace GodHand.Infrastructure.LinqParser
{
    public class OrderByExpressionParser
    {
        public static void Parse(Expression oe, ParserContext context)
        {
            OrderByExpressionParser.Parse(oe, context, null);
        }
        static void Parse(Expression oe, ParserContext context, string direction)
        {
            if (oe is MethodCallExpression)
            {
                var methodCallExp = oe as MethodCallExpression;
                var orderMethod = methodCallExp.Method.Name.ToLower();
                var orderExp = methodCallExp.Arguments[1];
                direction = orderMethod.EndsWith("descending") ? "DESC" : "ASC";
                OrderByExpressionParser.Parse(orderExp, context, direction);
            }
            else if (oe is UnaryExpression)
            {
                var operandExp = (oe as UnaryExpression).Operand;
                OrderByExpressionParser.Parse(operandExp, context, direction);
            }
            else if (oe is LambdaExpression)
            {
                var lambdaExp = oe as LambdaExpression;
                OrderByExpressionParser.Parse(lambdaExp.Body, context, direction);
            }
            else if (oe is MemberInitExpression)
            {
                var newExp = oe as MemberInitExpression;
                foreach (var b in newExp.Bindings)
                {
                    if (b is MemberAssignment)
                    {
                        var ma = (b as MemberAssignment);
                        var cName = ma.Member.Name;
                        var originalName = context.GetColumnName(cName);
                        context.AddSelectColumnName(originalName);
                        context.AddOrderBy(originalName, direction);
                    }
                }
            }
            else if (oe is NewExpression)
            {
                var newExp = oe as NewExpression;
                for (var i = 0; i < newExp.Arguments.Count; i++)
                {
                    var arg = newExp.Arguments[i];

                    if (arg is MemberExpression)
                    {
                        var member = newExp.Members[i];
                        var aliasName = member.Name;
                        var columnName = (arg as MemberExpression).Member.Name;
                        if (aliasName != columnName)
                        {
                            context.AddColumnMap(aliasName, columnName);
                        }
                        var originalName = context.GetColumnName(columnName);
                        context.AddSelectColumnName(originalName);
                        context.AddOrderBy(originalName, direction);
                    }
                }
            }
            else if (oe is MemberExpression)
            {
                var columnName = (oe as MemberExpression).Member.Name;
                var originalName = context.GetColumnName(columnName);
                context.AddSelectColumnName(originalName);
                context.AddOrderBy(originalName, direction);
            }
            else
            {
                throw new Exception("未能解析Orderby Expression.");
            }
        }
    }
}
