﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using System.Data;

namespace GodHand.Infrastructure.LinqParser
{
    [Serializable]
    public class QueryCommnClosure
    {
        public int Take { get; set; }
        public int Skip { get; set; }
        public bool NoLock { get; set; }
        public string MethodName { get; set; }
        public string TableName { get; set; }
        public List<string> SelectColumns { get; set; }
        //public List<SelectTypeConstructor> SelectTypes { get; set; }
        public Dictionary<string, string> OrderBys { get; set; }
        public List<string> GroupBys { get; set; }
        public List<string> WhereSQLs { get; set; }
        public string EvalNumericSQL { get; set; }
        public string TableAlias { get; set; }

        public QueryCommnClosure()
        {
            this.SelectColumns = new List<string>();
            //this.SelectTypes = new List<SelectTypeConstructor>();
            this.OrderBys = new Dictionary<string, string>();
            this.GroupBys = new List<string>();
            this.WhereSQLs = new List<string>();
        }

        public void Generate(ParserContext context)
        {
            var evalNumericSql = string.Empty;
            var countSql = string.Empty;
            var existSql = string.Empty;

            switch (this.MethodName)
            {
                case "sum": { this.OrderBys.Clear(); this.SelectColumns.Clear(); evalNumericSql = string.Format("SUM({0})", this.EvalNumericSQL); break; }
                case "max": { this.OrderBys.Clear(); this.SelectColumns.Clear(); evalNumericSql = string.Format("MAX({0})", this.EvalNumericSQL); break; }
                case "min": { this.OrderBys.Clear(); this.SelectColumns.Clear(); evalNumericSql = string.Format("MIN({0})", this.EvalNumericSQL); break; }
                case "average": { this.OrderBys.Clear(); this.SelectColumns.Clear(); evalNumericSql = string.Format("AVG({0})", this.EvalNumericSQL); break; }
                case "any":
                case "contains":
                    {
                        this.OrderBys.Clear();
                        if (this.SelectColumns.Count == 0) { this.SelectColumns = context.DefaultAllColumns; }
                        existSql = "SELECT (CASE WHEN EXISTS(SELECT NULL AS [EMPTY] FROM({0}) AS EMPTYTABLE) THEN 1 ELSE 0 END) AS [value]";
                        break;
                    }
                case "distinct": { this.SelectColumns.Clear(); break; }
                case "longcount":
                case "count": { this.OrderBys.Clear(); this.SelectColumns.Clear(); evalNumericSql = "COUNT(*)"; break; }
            }

            string orderbySql = string.Empty;
            if (this.OrderBys.Count > 0)
            {
                var orderbyNames = this.OrderBys.Select(o => string.Format("[{0}].[{1}] {2}", this.TableAlias, o.Key, o.Value)).ToArray();
                orderbySql = string.Concat("ORDER BY ", string.Join(",", orderbyNames));
            }
            string groupbySql = string.Empty;
            if (this.GroupBys.Count > 0)
            {
                groupbySql = string.Concat("GROUP BY ", string.Join(",", this.GroupBys.Select(c => string.Format("[{0}].[{1}]", this.TableAlias, c)).ToArray()));
            }
            string topSql = string.Empty;
            string rownum = string.Empty;
            var selectColumnSql = string.Join(",", this.SelectColumns.Select(s => string.Format("[{0}].[{1}]", this.TableAlias, s)).ToArray());
            if (string.IsNullOrWhiteSpace(evalNumericSql) && this.Take > 0)
            {
                if (Skip == 0) { topSql = string.Format("TOP {0}", this.Take); }
                else
                {
                    rownum = string.Format(",ROW_NUMBER() OVER ({0}) as [ROW_NUMBER]", orderbySql);
                    orderbySql = string.Empty;
                }
            }

            List<string> whereSqls = new List<string>();
            this.WhereSQLs.ForEach(ws => { whereSqls.Add(ws.ToString()); });

            string whereSql = string.Empty;
            if (whereSqls.Count > 0)
            {
                whereSql = string.Concat("WHERE ", string.Join(" AND ", whereSqls));
            }

            if (string.IsNullOrEmpty(context.SQL))
            {
                string noLockSql = this.NoLock ? "WITH(NOLOCK)" : string.Empty;

                context.SQL = string.Format("SELECT {0} {1}{2} {3} FROM [{4}] AS [{5}] {6} {7} {8} {9}", topSql, selectColumnSql, rownum, evalNumericSql, this.TableName == null ? context.TableAttribute.TableName : this.TableName, this.TableAlias, noLockSql, whereSql, groupbySql, orderbySql);
            }
            else
            {
                context.SQL = string.Format("SELECT {0} {1}{2} {3} FROM ({4}) AS [{5}] {6} {7} {8}", topSql, selectColumnSql, rownum, evalNumericSql, context.SQL, this.TableAlias, whereSql, groupbySql, orderbySql);
            }
            if (!string.IsNullOrEmpty(rownum))
            {
                var pageTableAlias = "p" + this.TableAlias;
                var pageSelectColumnSql = string.Join(",", this.SelectColumns.Select(s => string.Format("[{0}].[{1}]", pageTableAlias, s)).ToArray());
                context.SQL = string.Format("SELECT TOP {0} {1} FROM ({2}) AS [{3}] WHERE ([{3}].[ROW_NUMBER] > {4})", this.Take, pageSelectColumnSql, context.SQL, pageTableAlias, this.Skip);
            }
            if (!string.IsNullOrEmpty(existSql))
            {
                context.SQL = string.Format(existSql, context.SQL);
            }
        }
    }
}
