﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using System.Data;

namespace GodHand.Infrastructure.LinqParser
{
    public class QueryExpressionClosure
    {
        public int Take { get; set; }
        public int Skip { get; set; }
        public string MethodName { get; set; }
        public List<Expression> QuerySelectExpressions = new List<Expression>();
        public List<Expression> WhereExpressions = new List<Expression>();
        public List<Expression> OrderByExpressions = new List<Expression>();
        public List<Expression> GroupByExpressions = new List<Expression>();
        public Expression EvalNumericExpression = null;

        public QueryCommnClosure PrepareContext(ParserContext context)
        {
            var qcc = new QueryCommnClosure { Take = this.Take, Skip = this.Skip, MethodName = this.MethodName, TableAlias = context.TableAlias };

            context.ClearSelectColumns();
            context.ClearSelectTypes();
            context.ClearOrderBys();
            context.ClearGroupBys();

            this.QuerySelectExpressions.ForEach(se =>
            {
                if (se == null) { return; }
                SelectExpressionParser.Parse(se, context);
            });

            if (this.WhereExpressions.Count > 0)
            {
                var whereVisitor = new WhereExpressionVisitor();
                this.WhereExpressions.ForEach(we =>
                {
                    if (we == null) { return; }
                    whereVisitor.Parse(we, context);
                    qcc.WhereSQLs.Add(whereVisitor.WhereBuilder.ToString());
                });
            }

            if (this.EvalNumericExpression != null)
            {
                this.OrderByExpressions.Clear();
                var evalVisitor = new EvalNumericExpressionVisitor();
                evalVisitor.Parse(this.EvalNumericExpression, context);
                qcc.EvalNumericSQL = evalVisitor.WhereBuilder.ToString();
            }

            if (Utility.ToListMethods.Contains(qcc.MethodName) || Utility.FirstMethods.Contains(qcc.MethodName))
            {
                //如果没有Select方法 则选择所有列
                if (!context.HasSelectMethod) { qcc.SelectColumns.AddRange(context.DefaultAllColumns); }
                //如果没有选择列 则用上一个查询的选择列
                var selectColumns = context.GetSelectColumns();
                if (this.QuerySelectExpressions.Count > 0) { qcc.SelectColumns = selectColumns; }
                if (selectColumns.Count == 0 && context.LastSelectColumns != null) { qcc.SelectColumns.AddRange(context.LastSelectColumns); }

                //把本次的选择列 放入上下文对象中
                context.LastSelectColumns = qcc.SelectColumns;
            }

            this.OrderByExpressions.ForEach(se =>
            {
                if (se == null) { return; }
                OrderByExpressionParser.Parse(se, context);
            });
            qcc.OrderBys = context.GetOrderBys();
            if (qcc.OrderBys.Count == 0 && Utility.PageMethods.Contains(qcc.MethodName))
            {
                qcc.OrderBys = context.DefaultOrderBys;
                if (qcc.OrderBys.Count == 0) { throw new Exception("实体分页必须指定Orderby属性或指定一个默认的属性为主键。"); }
            }

            this.GroupByExpressions.ForEach(se =>
            {
                if (se == null) { return; }
                GroupByExpressionParser.Parse(se, context);
            });
            //group by 的时候只能查询group by的列
            var groupbys = context.GetGroupBys();
            if (groupbys.Count > 0)
            {
                qcc.SelectColumns.Clear();
                qcc.GroupBys = groupbys;
                qcc.GroupBys.ForEach(c => { qcc.SelectColumns.Add(c); });
                //orderby 也只能固定为groupby的列
                if (qcc.OrderBys.Count > 0)
                {
                    var removeOrderBys = new List<string>();

                    qcc.OrderBys.ForEach((o, i) =>
                    {
                        if (!qcc.GroupBys.Contains(o.Key)) { removeOrderBys.Add(o.Key); }
                    });
                    removeOrderBys.ForEach(r => { qcc.OrderBys.Remove(r); });
                }
            }
            context.IncreaseTableIndex();
            return qcc;
        }

        public QueryExpressionClosure Clone()
        {
            var me = new QueryExpressionClosure();
            me.Take = this.Take;
            me.Skip = this.Skip;
            me.MethodName = this.MethodName;
            me.QuerySelectExpressions = this.QuerySelectExpressions.ToList();
            me.WhereExpressions = this.WhereExpressions.ToList();
            me.OrderByExpressions = this.OrderByExpressions.ToList();
            me.GroupByExpressions = this.GroupByExpressions.ToList();
            me.EvalNumericExpression = this.EvalNumericExpression;
            return me;
        }
    }
}
