﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using System.Reflection;

namespace GodHand.Infrastructure.LinqParser
{
    public class SelectExpressionParser
    {
        public static void Parse(Expression se, ParserContext context)
        {
            if (se is UnaryExpression)
            {
                var operandExp = (se as UnaryExpression).Operand;
                SelectExpressionParser.Parse(operandExp, context);
            }
            else if (se is LambdaExpression)
            {
                var lambdaExp = se as LambdaExpression;
                SelectExpressionParser.Parse(lambdaExp.Body, context);
            }
            else if (se is MemberInitExpression)
            {
                var newExp = se as MemberInitExpression;
                var tctor = new SelectTypeConstructor { SelectType = newExp.Type, ConstructorInfo = newExp.NewExpression.Constructor };
                context.AddSelectType(tctor);
                foreach (var b in newExp.Bindings)
                {
                    if (b is MemberAssignment)
                    {
                        var ma = (b as MemberAssignment);
                        var cName = ma.Member.Name;
                        var originalName = context.GetColumnName(cName);
                        context.AddSelectColumnName(originalName);
                        tctor.ParamNames.Add(originalName);
                    }
                }
            }
            else if (se is NewExpression)
            {
                var newExp = se as NewExpression;
                var tctor = new SelectTypeConstructor { SelectType = newExp.Type, ConstructorInfo = newExp.Constructor };
                context.AddSelectType(tctor);
                for (var i = 0; i < newExp.Arguments.Count; i++)
                {
                    var arg = newExp.Arguments[i];

                    if (arg is MemberExpression)
                    {
                        var member = newExp.Members[i];
                        var aliasName = member.Name;
                        var columnName = (arg as MemberExpression).Member.Name;
                        if (aliasName != columnName)
                        {
                            context.AddColumnMap(aliasName, columnName);
                        }
                        var originalName = context.GetColumnName(columnName);
                        context.AddSelectColumnName(originalName);
                        tctor.ParamNames.Add(originalName);
                    }
                }
            }
            else if (se is MemberExpression)
            {
                var memberInfo=(se as MemberExpression).Member;
                Type selectType = null;
                switch (memberInfo.MemberType) {
                    case MemberTypes.Property: { selectType = ((PropertyInfo)memberInfo).PropertyType; break; }
                    case MemberTypes.Field: { selectType = ((FieldInfo)memberInfo).FieldType; break; }
                    default: { throw new Exception("未能解析select类型."); }
                }
                var columnName = memberInfo.Name;
                var originalName = context.GetColumnName(columnName);
                context.AddSelectColumnName(originalName);
                var tctor = new SelectTypeConstructor { SelectType = selectType };
                tctor.ParamNames.Add(originalName);
                context.AddSelectType(tctor);
            }
            else { throw new Exception("未能解析Select Expression."); }
        }
    }
}
