﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Linq.Dynamic;
using System.Linq.Expressions;
using System.Collections;
using System.Reflection;

namespace GodHand.Infrastructure.Repository
{
    public abstract class AbstractRepository<T> : IRepository<T> where T : class, new()
    {
        public IObjectContext Context { get; private set; }
        public AbstractRepository(IObjectContext context)
        {
            this.Context = context;
        }
        protected abstract IQueryable<T> GetQueryable();
        public abstract IRepository<T> AsTableName(string tableName);
        public abstract IRepository<T> AsNoLock();
        public abstract IRepository<T> AsNoTracking();
        public abstract bool New(T entity, bool INSERT_IDENTITY = false);
        public abstract bool Update(T entity);
        public abstract void Update(Expression<Func<T, bool>> exp, object obj);
        public abstract bool Delete(T t);
        public abstract void Delete(Expression<Func<T, bool>> exp);
        public abstract bool VirtualDelete(T t);
        public virtual T CreateModel()
        {
            return new T();
        }
        public virtual Guid NewGuid() { return Guid.NewGuid(); }
        public virtual decimal NewId() { return decimal.Zero; }

        public IEnumerator<T> GetEnumerator()
        {
            return GetQueryable().GetEnumerator();
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetQueryable().GetEnumerator();
        }

        public Type ElementType
        {
            get { return GetQueryable().ElementType; }
        }

        public Expression Expression
        {
            get { return GetQueryable().Expression; }
        }

        public IQueryProvider Provider
        {
            get { return GetQueryable().Provider; }
        }
    }
}
