﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;

namespace GodHand.Infrastructure.Repository
{
    public class ContextFactory
    {
        static Dictionary<string, Type> cache = new Dictionary<string, Type>();
        static object lockHelper = new object();
        public static IObjectContext Create(string name)
        {
            var contextElement = RepositoryContextConfiguration.GetContextConfiguration(name);
            if (contextElement == null) { throw new Exception(string.Format("未找到仓储上下文对象的配置项{0}。", name)); }
            var values = contextElement.ContextType.Split(';');
            var typeName = values.First();
            var assemblyName = values.Last();
            Type contextType = null;
            if (!cache.ContainsKey(name))
            {
                lock (lockHelper)
                {
                    if (!cache.ContainsKey(name))
                    {
                        var assembly = Assembly.Load(assemblyName);
                        if (assembly == null) { throw new Exception(string.Format("未找到程序集{0}失败。", assemblyName)); }
                        contextType = assembly.GetTypes().FirstOrDefault(c => c.FullName == typeName);
                        if (contextType == null) { throw new Exception(string.Format("未找到仓储上下文类型{0}。", typeName)); }
                        cache.Add(name, contextType);
                    }
                }
            }
            contextType = cache[name];
            var param = contextElement.GetParamValues();
            var context = Activator.CreateInstance(contextType, param) as IObjectContext;
            if (context == null) { throw new Exception(string.Format("未能创建上下文对象{0}", typeName)); }
            context.Name = name;
            return context;
        }
    }
}
