﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using System.Linq.Dynamic;
using System.Linq.Expressions;

namespace GodHand.Infrastructure.Repository
{
    public static class RepositoryExtension
    {
        public static IQueryable<T> Where<T>(this IQueryable<T> q, Criteria criteria) where T : class
        {
            if (criteria == null || criteria.Precidate.Count == 0) { return q; }

            var pres = new List<string>();
            var preValues = new List<object>();
            for (var i = 0; i < criteria.Precidate.Count; i++)
            {
                var exp = criteria.Precidate[i];
                var columnName = exp.ColumnName;
                var columnType = exp.ColumnType;
                var op = exp.Operator;
                var val = exp.Value;
                switch (op)
                {
                    case "null": { val = null; pres.Add(string.Format("{0}=@{1}", columnName, i)); break; }
                    case "notnull": { val = null; pres.Add(string.Format("{0}<>@{1}", columnName, i)); break; }
                    case "<>":
                    case "!=": { pres.Add(string.Format("{0}<>@{1}", columnName, i)); break; }
                    case "=": { pres.Add(string.Format("{0}=@{1}", columnName, i)); break; }
                    case "like":
                        {
                            if (columnType != "string") { throw new Exception("like操作符只支持string类型！"); }
                            pres.Add(string.Format("{0}.Contains(@{1})", columnName, i));
                            break;
                        }
                    case "%like":
                        {
                            if (columnType != "string") { throw new Exception("%like操作符只支持string类型！"); }
                            pres.Add(string.Format("{0}.StartWiths(@{1})", columnName, i));
                            break;
                        }
                    case "like%":
                        {
                            if (columnType != "string") { throw new Exception("like%操作符只支持string类型！"); }
                            pres.Add(string.Format("{0}.EndWiths(@{1})", columnName, i));
                            break;
                        }
                    case ">=":
                    case "<=":
                    case "<":
                    case ">":
                        {
                            switch (columnType)
                            {
                                case "long":
                                case "int":
                                case "float":
                                case "date":
                                case "decimal": { pres.Add(string.Format("{0}{1}@{2}", columnName, op, i)); break; }
                                default: { throw new Exception(string.Format(">操作符不支持{0}类型", columnType)); }
                            }
                            break;
                        }
                }
                switch (columnType)
                {
                    case "float": { preValues.Add(Convert.ToSingle(val)); break; }
                    case "long": { preValues.Add(Convert.ToInt64(val)); break; }
                    case "int": { preValues.Add(Convert.ToInt32(val)); break; }
                    case "decimal": { preValues.Add(Convert.ToDecimal(val)); break; }
                    case "bool": { preValues.Add(Convert.ToBoolean(val)); break; }
                    case "date": { preValues.Add(Convert.ToDateTime(val)); break; }
                    default: { preValues.Add(val); break; }
                }
            }
            var precidate = string.Join(" and ", pres.ToArray());
            var precidateValue = preValues.ToArray();
            if (!string.IsNullOrWhiteSpace(precidate))
            {
                q = q.Where(precidate, precidateValue);
            }
            return q;
        }
        public static IQueryable<T> Query<T>(this IQueryable<T> q, Criteria criteria) where T : class
        {
            if (criteria == null) { throw new Exception("分页Criteria不能为null。"); }
            if (criteria.Precidate.Count > 0)
            {
                q = q.Where(criteria);
            }
            if (criteria.OrderBy.Count == 0)
            {
                throw new Exception("Repository分页必须指明排序。");
            }
            q = q.OrderBy(string.Join(",", criteria.OrderBy.Select(c => string.Format("{0} {1}", c.Key, c.Value))), null);
            if (criteria.Start > 0)
            {
                q = q.Skip(criteria.Start);
            }
            if (criteria.Limit > 0)
            {
                q = q.Take(criteria.Limit);
            }
            return q;
        }
        public static PageResult<T> Page<T>(this IQueryable<T> q, Criteria criteria) where T : class
        {
            if (criteria == null) { throw new Exception("分页Criteria不能为null。"); }
            if (criteria.Precidate.Count > 0)
            {
                q = q.Where(criteria);
            }
            var totalCount = q.Count();
            List<T> list = null;
            if (totalCount > 0)
            {
                if (criteria.OrderBy.Count == 0)
                {
                    throw new Exception("Repository分页必须指明排序。");
                }
                var str_orderby = string.Join(",", criteria.OrderBy.Select(c => string.Format("{0} {1}", c.Key, c.Value)));
                q = q.OrderBy(str_orderby, null);
                if (criteria.Start > 0)
                {
                    q = q.Skip(criteria.Start);
                }
                if (criteria.Limit > 0)
                {
                    q = q.Take(criteria.Limit);
                }
                list = q.ToList();
            }
            else
            {
                list = new List<T>();
            }
            //LogManager.GetLog("ObjectQuery").Info(((System.Data.Objects.ObjectQuery)q).ToTraceString());
            //System.Diagnostics.Debug.WriteLine(((System.Data.Objects.ObjectQuery)q).ToTraceString());
            return new PageResult<T> { List = list, TotalCount = totalCount };
        }
        public static IRepository<T> GetRepository<T>(this IObjectContext context) where T : class, new()
        {
            return RepositoryFactory.Create<T>(context);
        }
        public static PageResult<T> Query<T>(this IDomainServiceCRUD<T> service, Criteria criteria, string contextName = null) where T : class, new()
        {
            using (var context = ContextFactory.Create(contextName == null ? service.ContextName : contextName))
            {
                var r = context.GetRepository<T>();
                return r.Page(criteria);
            }
        }
        public static PageResult<T> Query<T>(this IDomainServiceCRUD<T> service, Expression<Func<T, bool>> predicate, Criteria criteria, string contextName = null) where T : class, new()
        {
            using (var context = ContextFactory.Create(contextName == null ? service.ContextName : contextName))
            {
                var r = context.GetRepository<T>().Where(predicate);
                return r.Page(criteria);
            }
        }
        public static T Create<T>(this IDomainServiceCRUD<T> service, T tdto, string contextName = null) where T : class,IEntity, new()
        {
            using (var context = ContextFactory.Create(contextName == null ? service.ContextName : contextName))
            {
                var r = context.GetRepository<T>();
                var instance = r.CreateModel();
                instance.Copy(tdto);
                instance.Validate(context);
                r.New(instance);
                context.SaveChanges();
                tdto.Copy(instance);
                return instance;
            }
        }
        public static T Update<T>(this IDomainServiceCRUD<T> service, T tdto, Func<T, bool> selectPrecidate, string contextName = null) where T : class,IEntity, new()
        {
            using (var context = ContextFactory.Create(contextName == null ? service.ContextName : contextName))
            {
                var r = context.GetRepository<T>();
                var instance = r.FirstOrDefault(selectPrecidate);
                if (instance == null) { throw new Exception("未找到数据" + typeof(T).Name); }
                instance.Copy(tdto);
                instance.Validate(context);
                r.Update(instance);
                context.SaveChanges();
                return instance;
            }
        }
        public static T Delete<T>(this IDomainServiceCRUD<T> service, Func<T, bool> precidate, string contextName = null) where T : class,IEntity, new()
        {
            using (var context = ContextFactory.Create(contextName == null ? service.ContextName : contextName))
            {
                var r = context.GetRepository<T>();
                var instance = r.FirstOrDefault(precidate);
                instance.Dirty(context);
                r.Delete(instance);
                context.SaveChanges();
                return instance;
            }
        }
        public static T VirtualDelete<T>(this IDomainServiceCRUD<T> service, Func<T, bool> precidate, string contextName = null) where T : class,IEntity, new()
        {
            using (var context = ContextFactory.Create(contextName == null ? service.ContextName : contextName))
            {
                var r = context.GetRepository<T>();
                var instance = r.FirstOrDefault(precidate);
                if (instance == null) { throw new Exception("未找到数据" + typeof(T).Name); }
                instance.Dirty(context);
                r.VirtualDelete(instance);
                context.SaveChanges();
                return instance;
            }
        }
    }
}
