﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;

namespace GodHand.Infrastructure.Repository
{
    public class RepositoryFactory
    {
        static Dictionary<string, Type> cache = new Dictionary<string, Type>();
        static object lockHelper = new object();

        public static IRepository<T> Create<T>(IObjectContext context) where T : class, new()
        {
            var repositoryTypeStr = RepositoryContextConfiguration.GetContextConfiguration(context.Name).RepositoryType;
            var repositoryKey = typeof(T).Name + repositoryTypeStr;
            var values = repositoryTypeStr.Split(';');
            var typeName = values.First();
            var assemblyName = values.Last();

            Type repositoryTypeG = null;
            if (!cache.ContainsKey(repositoryKey))
            {
                lock (lockHelper)
                {
                    if (!cache.ContainsKey(repositoryKey))
                    {
                        var assembly = Assembly.Load(assemblyName);
                        if (assembly == null) { throw new Exception(string.Format("未找到程序集{0}失败。", assemblyName)); }
                        var repositoryType = assembly.GetTypes().FirstOrDefault(c => c.IsGenericType && c.FullName.StartsWith(typeName));
                        if (repositoryType == null) { throw new Exception(string.Format("未找到仓储类型{0}", typeName)); }

                        repositoryTypeG = repositoryType.MakeGenericType(typeof(T));
                        cache.Add(repositoryKey, repositoryTypeG);
                    }
                }
            }
            repositoryTypeG = cache[repositoryKey];
            var repository = Activator.CreateInstance(repositoryTypeG, context) as IRepository<T>;
            if (repository == null) { throw new Exception(string.Format("未能创建仓储对象{0}", typeName)); }
            return repository;
        }
    }
}
